const WebSocket = require('ws');
const Upvote = require('./Upvote');
const EventEmitter = require('node:events');

class Client extends EventEmitter {
    constructor(options = {}) {
        super()
        this.ws = null;
        this.trackOnly = null;
        if (Array.isArray(options.trackOnly)) this.trackOnly = options.trackOnly;
        if (options.trackOnly && !Array.isArray(options.trackOnly)) throw new TypeError("El trackOnly debe ser un array con IDs de los bots de los cuales quieres tener el seguimiento de votos.");

        if (!options.apiToken) {
            throw new TypeError("El token de la API no fue dada.")
        }
        this.sessionId = null;
        this.apiToken = options.apiToken;
        this._heartbeatInterval = null;
    }
    async connect(options = {}) {
        let token = this.apiToken;
        let trackOnly = this.trackOnly;
        return new Promise((resolve, reject) => {
            const reconnectTimeout = 10000;

            const ws = new WebSocket("wss://auralist.ml/gateway");
            this.ws = ws;
            ws.on('open', () => {
                this.emit('debug', '[ABL - DEBUG] - Conexión abierta')
                ws.send(JSON.stringify({
                    type: "IDENTIFY",
                    data: {
                        api_token: token,
                        track_only: trackOnly
                    }
                }));
                ws.on('message', (data) => {
                    data = JSON.parse(data);
                    if (data.code == 3001) {
                        this.emit('debug', "[ABL - DEBUG] - Se conectó correctamente al servidor.");
                        if (options.reconnect != true) this.emit('ready', undefined);
                        this.sessionId = data.data.sessionId;
                        this._heartbeatInterval = setInterval(() => {
                            this.emit('debug', "[ABL - DEBUG] - Enviando ping...")
                            ws.send(JSON.stringify({
                                type: "PING",
                                data: {
                                    sessionId: this.sessionId
                                }
                            }));
                            this.emit('debug', "[ABL - DEBUG] - Ping enviado.")
                        }, 30000);
                        resolve(token);
                    }
                    if (data.code == 5000) { // Event 
                        if (data.type == "UPVOTE") {
                            let upvoteData = new Upvote(data.data);
                            this.emit("vote", upvoteData);
                        }
                    }
                    if (data.code == 1500) {
                        reject(new Error(data.message));
                    }
                });
            });
            ws.on('close', (code, reason) => {
                console.log(code, reason.toString())
                if (code == 1006) { // Unexpected disconnection;
                    this.emit('debug', '[ABL - DEBUG] - Se desconectó de manera inesperada del servidor.');
                    this.emit('debug', '[ABL - DEBUG] - Tratando de volverse a conectar en 10s.');

                    const reconnect = () => {
                        setTimeout(() => {
                            this.connect({ reconnect: true, sessionId: this.sessionId })
                                .then(e => this.emit('debug', "[ABL - DEBUG] - Reconexión éxitosa."))
                                .catch(e => {
                                    this.emit('debug', '[ABL - DEBUG] - No se pudo reconectar, reintentando... ' + e);
                                    reconnect();
                                });
                        }, reconnectTimeout);
                    }
                    reconnect()
                } else {
                    throw new Error(`[AuraList]: Disconnection Code: ${code}. Reason: ${reason.toString()}`)
                }
            });
            ws.on('error', (e) => {
                if (e)
                    throw e;
                if (!this.sessionId) ws.close(1001);
                clearInterval(this._heartbeatInterval);
                ws.close(1006);
            })
        })
    }
    async destroy() {
        return this.ws.close(1001, "Chaooooo");
    }
}

module.exports = Client;